#!/bin/bash
#
# kill-thread.sh
#  Simple mysql connection killing script
#
# Changelog
#  2014-03-26 - Nathan Isburgh <nathan@edgecloud.com>
#             * Initial version
#

# User configurable variables
DEBUG=1

#
# END OF CONFIGURATION VARIABLES
# NO MORE EDITS BELOW THIS LINE
#

# Functions
fail() {
  cleanup
  ECODE=${1:-1}
  EMSG=$2
  if [ -n "$EMSG" ]
  then
    printf "%s\n" $EMSG > /dev/stderr
  fi
  exit $ECODE
}

succeed() {
  cleanup
  exit 0
}

cleanup() {
  # Close log files, shutdown connections, etc..
  return
}

debug() {
  if [ $DEBUG -eq 0 ] 
  then
    return
  fi
  printf " >>> %s %s\n" "$(date)" "$1" > /dev/stderr
}

usage() {
  cat << eof
Usage:
  $0 [-u user] [-h host] [-d db] [-c command]
  Note: The filtering options are AND-ed together
   -u user    Kill connections by user
   -h host    Kill connections by host
   -d db      Kill connections by database
   -c command Kill connections by command
eof
}

kill_threads() {
  mysql -N -e "sHOW fULl prOCEssLIsT" | fgrep -v "sHOW fULl prOCEssLIsT" | awk -F'\t' "\$2 ~ /$FILTER_USER/ && \$3 ~ /$FILTER_HOST/ && \$4 ~ /$FILTER_DB/ && \$8 ~ /$FILTER_COMMAND/ {print \$1}" | xargs -n 1 mysqladmin kill
}

# Main body

# Add hook to call cleanup on exit automagically
trap cleanup TERM QUIT ABRT INT EXIT

if [ $# -lt 2 ]
then
  echo "Please supply at least one filter" > /dev/stderr
  usage
  fail
fi

FILTER_USER=""
FILTER_HOST=""
FILTER_DB=""
FILTER_COMMAND=""

while getopts ":u:h:d:c:" opt; do
  case $opt in
    u)
      FILTER_USER=$OPTARG
      ;;
    h)
      FILTER_HOST=$OPTARG
      ;;
    d)
      FILTER_DB=$OPTARG
      ;;
    c)
      FILTER_COMMAND=$OPTARG
      ;;
    \?)
      echo "Invalid option: -$OPTARG" > /dev/stderr
      usage
      fail 1
      ;;
    :)
      echo "Option -$OPTARG requires an argument." > /dev/stderr
      usage
      fail 1
      ;;
  esac
done

kill_threads

succeed
